
-- | Define several constants used throughout all libraries depending on
-- Biobase.

module Biobase.Constants where



-- * energy "Infinities" for Int's

-- | maxBound on Int is problematic, if we try something like maxBound +
-- maxBound which happens very often. 'eInf' is used for settings like lookup
-- tables, etc while comparisons are against 'eMax'.
-- Both values need to be small enough not to overflow in typical situations.

eInf :: Int
eInf = 10000000

eMax :: Int
eMax = eInf `div` 10



-- * floating infinity

-- here, we choose a real infinity and let the ieee handle it

fInf :: Double
fInf = 1/0


-- * some constants, required for scaling and Boltzmann



-- Kelvin / Celsius conversion constant

kelvinC0 = 273.15

-- universal gas constant

gasconst = 1.98717 -- cal/K
